@@ -16,39 +16,126 @@
 #
 #  High level module for managing the lookup of R objects.
 #
-#  $Id: rpy.py,v 1.25 2005/07/27 17:56:54 warnes Exp $
+#  $Id: rpy.py 387 2007-11-30 00:57:41Z warnes $
 #
 #
 from __future__ import nested_scopes
 
+from rpy_options import rpy_options
+
 import rpy_io
 import UserDict
-import time, os, sys
+import time, os, sys, atexit, glob
 import rpy_tools
 
 # If we cannot import Numeric, it should have been detected at
 # installation time and RPy should have been compiled properly. So,
 # don't complain.
 try:
-    from Numeric import *
-    HAS_NUMERIC = 1
+    from numpy import *
+    HAS_NUMERIC = 3
 except ImportError:
-    HAS_NUMERIC = 0
-    pass
+    try:
+        from Numeric import *
+        HAS_NUMERIC = 1
+    except ImportError:
+        HAS_NUMERIC = 0
+        pass
 
-RHOME = rpy_tools.get_R_HOME()
-RVERSION  = rpy_tools.get_R_VERSION(RHOME)
-RVER = rpy_tools.get_R_VERSION_CODE(RVERSION)
 
-if sys.platform == 'win32':
-    # Load the R dll using the explicit path
-    import win32api, win32con
-    Rlib = os.path.join( RHOME, 'bin', 'R.dll')
-    win32api.LoadLibraryEx( Rlib, 0,
-                            win32con.LOAD_WITH_ALTERED_SEARCH_PATH )
+# Get necessary paths and version information
+RHOME=os.environ.get('RHOME',None)
     
-# loading the version of rpy that matches the verison of R we're using
-exec("import _rpy%s as _rpy" % RVER)
+if not RHOME:
+    if rpy_options['RHOME'] is not None:
+        RHOME = rpy_options['RHOME']
+    else:
+        RHOME = rpy_tools.get_R_HOME()
+
+if rpy_options['RVERSION'] is not None:
+    RVERSION = rpy_options['RVERSION']
+else:
+    RVERSION = rpy_tools.get_R_VERSION(RHOME)
+
+if rpy_options['RVER'] is not None:
+    RVER = rpy_options['RVER']
+else:
+    RVER = rpy_tools.get_R_VERSION_CODE(RVERSION)
+
+if rpy_options['RUSER'] is not None:
+    RUSER = rpy_options['RUSER']
+else:
+    RUSER = rpy_tools.get_R_USER()
+
+VERBOSE=rpy_options['VERBOSE']
+
+if VERBOSE:
+    print "RHOME=",RHOME
+    print "RVERSION=",RVERSION
+    print "RVER=",RVER
+    print "RUSER=",RUSER
+
+# Push these into the environment for rpymodule to pick up
+os.environ['RPY_RHOME']=RHOME
+os.environ['RPY_RVERSION']=RVERSION
+os.environ['RPY_RVER']=RVER
+os.environ['RPY_RUSER']=RUSER
+
+# Push R_HOME into the environment for the R shared library/DLL to pick up
+os.environ['R_HOME']=RHOME
+
+if sys.platform=='win32':
+    import win32api
+    os.environ['PATH'] += ';' + os.path.join(RHOME,'bin')
+    os.environ['PATH'] += ';' + os.path.join(RHOME,'modules')
+    os.environ['PATH'] += ';' + os.path.join(RHOME,'lib')
+
+    # Load the R dll using the explicit path
+    # First try the bin dir:
+    Rlib = os.path.join( RHOME, 'bin', 'R.dll')
+    # Then the lib dir:
+    if not os.path.exists(Rlib):
+        Rlib = os.path.join( RHOME, 'lib', 'R.dll')
+    # Otherwise fail out!
+    if not os.path.exists(Rlib):
+        raise RuntimeError("Unable to locate R.dll within %s" % RHOME)
+
+    if VERBOSE:
+        print "Loading the R DLL %s .." % Rlib,
+        sys.stdout.flush()
+    win32api.LoadLibrary( Rlib )
+    if VERBOSE:
+        print "Done."
+        sys.stdout.flush()
+
+
+# load the version of rpy that matches the verison of R we're using
+if VERBOSE:
+    print "Loading Rpy version %s .." % RVER,
+    sys.stdout.flush()
+
+
+try:
+  command = "import _rpy%s as _rpy" % RVER
+  exec(command)
+except Exception, e:
+  raise RuntimeError( str(e) + 
+      """
+
+      RPy module can not be imported. Please check if your rpy
+      installation supports R %s. If you have multiple R versions
+      installed, you may need to set RHOME before importing rpy. For
+      example:
+  
+      >>> from rpy_options import set_options
+      >>> set_options(RHOME='c:/progra~1/r/rw2011/')
+      >>> from rpy import *
+      
+      """ % RVERSION)
+
+if VERBOSE:
+    print "Done."
+    sys.stdout.flush()
 
 # Version
 from rpy_version import rpy_version
@@ -81,8 +168,20 @@ def set_default_mode(mode):
 def get_default_mode():
     return _rpy.get_mode()
 
-# The new exception
-RException = _rpy.RException
+# Three new exceptions
+# base exception
+RPyException = _rpy.RPy_Exception;
+
+# R <-> Python conversion exception
+RPyTypeConversionException = _rpy.RPy_TypeConversionException;
+
+# Exception raised by R
+RPyRException = _rpy.RPy_RException
+
+# for backwards compatibility
+RException = RPyException
+
+
 
 # I/O setters
 set_rpy_output = _rpy.set_output
@@ -129,9 +228,11 @@ def start_r_eventloop():
 
 def stop_r_eventloop():
     _r_events_running.clear()
-    _r_thread.join()
+    if _r_thread:
+        _r_thread.join()
 
 if sys.platform != 'win32':
+    atexit.register(stop_r_eventloop)
     start_r_eventloop()
 
 
@@ -167,6 +268,9 @@ proc_table = Dict_With_Mode(_rpy.__proc_table__)
 class R:
 
     def __init__(self):
+        if rpy_options['VERBOSE']:
+            print "Creating the R object 'r' ..",
+            sys.stdout.flush()
         _rpy.r_init(HAS_NUMERIC);
         _rpy.set_mode(NO_DEFAULT)
         
@@ -180,6 +284,14 @@ class R:
         self.NAN = self('as.double(NA)')
         self.helpfun = with_mode(NO_CONVERSION, self.__getitem__('help'))
         self.help = self.__help__  # override r.help()
+
+        # workaround for plotting bug under win32
+        if sys.platform == 'win32':
+            self('options(windowsBuffered=FALSE)')
+
+        if rpy_options['VERBOSE']:
+            print " Done"
+            sys.stdout.flush()
         
     def __getattr__(self, name):
         if name.startswith('__') and name.endswith('__'): return object.__getattr__(name)
@@ -208,7 +320,7 @@ class R:
         self.print_(helpobj)
 
     def __repr__(self):
-        Rver = self.__getitem__('R.version.string').as_py()
+        Rver = self.__getitem__('R_version_string')
         return "RPy version %s [%s]" % (rpy_version, Rver)
 
     def __str__(self):
@@ -226,3 +338,4 @@ r = R()
 r.options(show_error_messages=0)
 
 # That's all...
+
