@@ -1,5 +1,5 @@
 /*
- * $Id: rpymodule.c 299 2006-03-22 22:13:54Z warnes $
+ * $Id: rpymodule.c 384 2007-11-30 00:40:17Z warnes $
  * Implementation of the module '_rpy' and the 'Robj' type.
  */
 
@@ -39,6 +39,7 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#define CSTACK_DEFNS
 #include "RPy.h"
 
 #define NONAMELESSUNION
@@ -46,9 +47,9 @@
 #include <stdlib.h>
 #include <string.h>
 
-/* Flag indicating whether Numeric is available in this session
+/* Flag indicating whether Numpy/Numeric is available in this session
  *
- * This is necessary Numeric may not available at run time, even if
+ * This is necessary since Numpy/Numeric may not available at run time, even if
  * it was available at compile time.
 */
 static int use_numeric=0;
@@ -73,7 +74,9 @@ static PyObject *class_table;
 static PyObject *proc_table;
 static int default_mode;
 static PyObject *r_lock;
-PyObject *RPyExc_Exception;
+PyObject *RPy_Exception;
+PyObject *RPy_TypeConversionException;
+PyObject *RPy_RException;
 
 static char RHOME[BUFSIZ];
 static char RVERSION[BUFSIZ];
@@ -181,6 +184,8 @@ type_to_int(PyObject *obj)
     return ANY_T;
   else if (PyString_Check(obj))
     return STRING_T;
+  else if (PyUnicode_Check(obj))
+    return STRING_T;
   else if (Robj_Check(obj))
     return ROBJ_T;
   else
@@ -221,21 +226,31 @@ seq_to_R(PyObject *obj)
   PROTECT(robj = NEW_LIST(len));
 
   state = -1;
-  for (i=0; i<len; i++) {
-    if (!(it = PySequence_GetItem(obj, i)))
-      goto exception;
-
-    if (state < 0)
-      state = type_to_int(it);
-    else
-      state = fsm[state][type_to_int(it)];
-
-    if (!(rit = to_Robj(it)))
-      goto exception;
-    
-    SET_VECTOR_ELT(robj, i, rit);
-    Py_XDECREF(it);
-  }
+  for (i=0; i<len; i++) 
+    {
+      it = PySequence_GetItem(obj, i);
+      if (it==NULL)
+	{
+	  UNPROTECT(1);
+	  return NULL;
+	}
+      
+      if (state < 0)
+	state = type_to_int(it);
+      else
+	state = fsm[state][type_to_int(it)];
+      
+      rit = to_Robj(it);
+      if (rit==NULL || PyErr_Occurred()	)
+	{
+	  Py_XDECREF(it);
+	  UNPROTECT(1);
+	  return NULL;
+	}
+      
+      SET_VECTOR_ELT(robj, i, rit);
+      Py_XDECREF(it);
+    }
 
   switch(state)
     {
@@ -260,11 +275,6 @@ seq_to_R(PyObject *obj)
   
   UNPROTECT(1);
   return robj;
-
-exception:
-  Py_XDECREF(it);
-  UNPROTECT(1);
-  return NULL;
 }
 
 /* Make a R named list or vector from a Python dictionary */
@@ -284,68 +294,191 @@ dict_to_R(PyObject *obj)
     return NULL;
   if (!(values = PyMapping_Values(obj)))
     return NULL;
-  
-  if (!(robj = seq_to_R(values)))
-    goto fail;
-  if (!(names = seq_to_R(keys)))
-    goto fail;
+
+  robj = seq_to_R(values);
+  names = seq_to_R(keys);
+  if ( robj==NULL || robj==NULL )
+    {
+      Py_DECREF(keys);
+      Py_DECREF(values);
+      return NULL;
+    }
+
   PROTECT(robj);
   SET_NAMES(robj, names);
+
   Py_DECREF(keys);
   Py_DECREF(values);
   UNPROTECT(1);
 
   return robj;
-
- fail:
-  Py_DECREF(keys);
-  Py_DECREF(values);
-  return NULL;
 }
 
 #ifdef WITH_NUMERIC
-/* Convert a Numeric array to a R array */
+/* Convert a Numpy/Numeric array to a R array */
 SEXP
 to_Rarray(PyObject *o)
 {
   PyObject *pytl, *nobj;
   PyArrayObject *obj;
   SEXP Rdims, tRdims, Rarray, e;
-  int *dims, i;
-  long tl;
+  n_intp *dims;
+  int i;
+  int type;
+  long size;
 
   obj = (PyArrayObject *)o;
   dims = obj->dimensions;
-  tl = 1;
-  PROTECT(Rdims = allocVector(INTSXP, obj->nd));
-  PROTECT(tRdims = allocVector(INTSXP, obj->nd));
-  for (i=0; i<obj->nd; i++) {
-    if (dims[i] == 0) {
-      UNPROTECT(2);
-      return R_NilValue;
+  type = obj->descr->type_num;
+  size = PyArray_Size( (PyObject*) obj);
+
+  /* Handle a vector without dimensions, just length */
+  if(obj->nd==0)
+    {
+      PROTECT(Rdims = allocVector(INTSXP, 1));
+      PROTECT(tRdims = allocVector(INTSXP, 1));
+      INTEGER(Rdims)[0] = size;
+      INTEGER(tRdims)[0] = size;
     }
-    tl *= dims[i];
-    INTEGER(Rdims)[i] = dims[(obj->nd)-i-1];
-    INTEGER(tRdims)[i] = (obj->nd)-i;
-  }
+  else
+    {
+      PROTECT(Rdims = allocVector(INTSXP, obj->nd));
+      PROTECT(tRdims = allocVector(INTSXP, obj->nd));
 
-/*   if (!(pytl = PyList_New(1))) */
-/*     goto exception; */
-/*   if (!(it = PyInt_FromLong(tl))) */
-/*     goto exception; */
-/*   if (PyList_SetItem(pytl, 0, it) < 0) */
-/*     goto exception; */
+      for (i=0; i<obj->nd; i++) 
+	{
+	  if (dims[i] == 0) 
+	    {
+	      UNPROTECT(2);
+	      return R_NilValue;
+	    }
+	  INTEGER(Rdims)[i] = dims[(obj->nd)-i-1];
+	  INTEGER(tRdims)[i] = (obj->nd)-i;
+	}
+    }
 
-  pytl = Py_BuildValue("[i]", tl);
-  obj = (PyArrayObject *)PyArray_ContiguousFromObject((PyObject *)obj,
-                                                      PyArray_NOTYPE, 0, 0);
+    switch(type)
+    {
+
+    /*******************/
+    /* String Variants */
+    /*******************/
+    /* TODO: Add proper handling of NumPy character arrays.  
+             The following code DOES NOT WORK: 
+
+	     #if WITH_NUMERIC==3 
+	     case PyArray_UNICODE:
+	     case PyArray_STRING:
+	     case PyArray_CHAR:
+	       obj = (PyArrayObject *)PyArray_ContiguousFromObject((PyObject *)obj,
+							  PyArray_STRING, 0, 0);
+	     #endif
+
+      The problem is that the PyArray call throws an exception,
+      presumably because we haven't given a width specifier.   
+      
+      NumPy strings are fixed-width, and may not be null terminated.  R only handles
+      null terminated (varying width) strings.  We need a separate
+      code path to handle this, as it requires quite different
+      handling than the numeric arrays dealt with below.
+    */
+
+
+    /******************************************/
+    /* All complex to (double,double) complex */
+    /******************************************/
+
+#if WITH_NUMERIC==1       /* Numeric */
+    case PyArray_CFLOAT:
+    case PyArray_CDOUBLE:
+#else                        /* NumPy */
+    case PyArray_COMPLEX64:
+    case PyArray_COMPLEX128:
+#endif
+      obj = (PyArrayObject *)PyArray_ContiguousFromObject((PyObject *)obj,
+							  PyArray_CDOUBLE, 0, 0);
+      break; 
+
+
+    /**********************************************************************************/
+    /* Convert all integers to platform integer (except 64 bit int on 32 bit platforms) */
+    /************************************************************************************/
+
+#if WITH_NUMERIC==1      /* Numeric */
+    case PyArray_UBYTE:
+    case PyArray_SBYTE:
+    case PyArray_SHORT:
+    case PyArray_INT:
+    case PyArray_LONG:
+      obj = (PyArrayObject *)PyArray_ContiguousFromObject((PyObject *)obj,
+							  PyArray_INT, 0, 0);
+      break;
+#else                    /* NumPy */
+    case PyArray_BOOL:
+    case PyArray_INT8:
+    case PyArray_UINT8: 
+    case PyArray_INT16:
+    case PyArray_UINT16:
+    case PyArray_INT32:
+    case PyArray_UINT32:
+#if PyArray_INT==PyArray_INT64 /* 64 bit platform */
+    case PyArray_INT64:
+    case PyArray_UINT64:
+#else
+      obj = (PyArrayObject *)PyArray_ContiguousFromObject((PyObject *)obj,
+							  PyArray_INT, 0, 0);
+      break;
+#endif
+#endif
+
+    /**************************************************/
+    /* All floats (and over-sized integers) to double */
+    /**************************************************/
+#if WITH_NUMERIC==1       /* Numeric */
+    case PyArray_FLOAT:
+    case PyArray_DOUBLE:
+#else                     /* NumPy */
+    case PyArray_FLOAT32:
+    case PyArray_FLOAT64:
+#if PyArray_INT!=PyArray_INT64 /* 32 bit platform */
+    case PyArray_INT64:
+    case PyArray_UINT64:
+#endif
+#endif
+      obj = (PyArrayObject *)PyArray_ContiguousFromObject((PyObject *)obj,
+							  PyArray_DOUBLE, 0, 0);
+      break;
+
+    default:
+      UNPROTECT(2);
+      PyErr_Format(RPy_TypeConversionException, 
+		   "Numeric/NumPy arrays containing %s are not supported.", 
+		   obj->ob_type->tp_name);
+      return R_NilValue;
+      break;
+    }
+
+
+  pytl = Py_BuildValue("[i]", size);
   nobj = PyArray_Reshape(obj, pytl);
   Py_XDECREF(pytl);
   Py_XDECREF(obj);
-  if (!nobj)
-    goto exception;
+
+  if (nobj == NULL) 
+    {
+      UNPROTECT(2);
+      return R_NilValue;
+    }
+   
 
   PROTECT(Rarray = seq_to_R(nobj));
+  if (Rarray == NULL)
+    {
+      UNPROTECT(3); 
+      return R_NilValue;
+    }
+
+
   Py_XDECREF(nobj);
   SET_DIM(Rarray, Rdims);
   
@@ -357,10 +490,6 @@ to_Rarray(PyObject *o)
 
   UNPROTECT(5);
   return Rarray;
-
- exception:
-  UNPROTECT(2);
-  return NULL;
 }
 #endif
 
@@ -373,9 +502,10 @@ to_Robj(PyObject *obj)
   SEXP robj;
   Py_complex c;
   PyObject *to_r_meth;
+  PyObject *tempObj;
   int do_decref = 0;
 
-  if (!obj)
+  if (obj==NULL)
     return NULL;
 
   if (obj == Py_None) {
@@ -386,7 +516,7 @@ to_Robj(PyObject *obj)
   if (to_r_meth) {
     obj = PyObject_CallObject(to_r_meth, NULL);
     Py_DECREF(to_r_meth);
-    if (!obj)
+    if (obj==NULL)
       return NULL;
     do_decref = 1;
   }
@@ -419,13 +549,28 @@ to_Robj(PyObject *obj)
       COMPLEX_DATA(robj)[0].r = c.real;
       COMPLEX_DATA(robj)[0].i = c.imag;
     }
+  else if (PyUnicode_Check(obj))
+    {
+      /** Handle Unicode Strings.
+       *
+       * Ideally:  Python Unicode -> R Unicode, 
+       *
+       * Unfortunately, the R documentation is not forthcoming on how
+       * to accomplish this 
+       *
+       * So, for the moment:  
+       *     python Unicode -> Python ASCII -> ordinary string -> R string  
+       *
+       */
+      PROTECT(robj = NEW_STRING(1));
+      SET_STRING_ELT(robj, 0, COPY_TO_USER_STRING(PyString_AsString(PyUnicode_AsASCIIString(obj))));
+    }
   else if (PyString_Check(obj)) 
     {
       PROTECT(robj = NEW_STRING(1));
       SET_STRING_ELT(robj, 0, COPY_TO_USER_STRING(PyString_AsString(obj)));
     }
 #ifdef WITH_NUMERIC
-  //else if (PyArray_Check(obj)) 
   else if (use_numeric && PyArray_Check(obj)) 
     {
       PROTECT(robj = to_Rarray(obj));
@@ -437,15 +582,26 @@ to_Robj(PyObject *obj)
       PROTECT(robj = seq_to_R(obj));      /* No labels */
     }
   else if ((PyMapping_Check(obj)) &&
-             (PyMapping_Size(obj) >= 0)) 
+            (PyMapping_Size(obj) >= 0)) 
     {
       PROTECT(robj = dict_to_R(obj));
     }
-  else 
+  else if (PyNumber_Check(obj)) /* generic number interface */
     {
-      PyErr_Format(RPyExc_Exception, "cannot convert from type '%s'",
+      tempObj = PyNumber_Float(obj); 
+      if(!tempObj) goto error;
+
+      PROTECT(robj = NEW_NUMERIC(1));
+      NUMERIC_DATA(robj)[0] = PyFloat_AsDouble(tempObj);
+      Py_DECREF(tempObj);
+    }
+  else
+    {
+    error:
+      PyErr_Format(RPy_TypeConversionException, 
+		   "cannot convert from type '%s'",
                    obj->ob_type->tp_name);
-      PROTECT(robj = NULL);       /* Protected to avoid stack inbalance */
+      PROTECT(robj = NULL);    /* Protected to avoid stack inbalance */
     }
 
   if (do_decref) 
@@ -462,7 +618,7 @@ to_PyDict(PyObject *obj, SEXP names)
 {
   int len, i;
   PyObject *it, *dict;
-  char *name;
+  const char *name;
 
   if ((len = PySequence_Length(obj)) < 0)
     return NULL;
@@ -547,7 +703,7 @@ to_PyNumericArray(PyObject *seq, SEXP dim)
   PyObject *array, *ret, *dims, *it;
   int l, i, j;
 
-  array = PyArray_ContiguousFromObject(seq, PyArray_NOTYPE, 0,0);
+  array = PyArray_ContiguousFromObject(seq, PyArray_DOUBLE, 0,0);
   if (!array)
     return NULL;
 
@@ -610,7 +766,7 @@ to_Pyobj_vector(SEXP robj, PyObject **obj, int mode)
   PyObject *it, *tmp;
   SEXP names, dim;
   int len, *integers, i, type;
-  char *strings, *thislevel;
+  const char *strings, *thislevel;
   double *reals;
   Rcomplex *complexes;
 #ifdef WITH_NUMERIC
@@ -886,7 +1042,7 @@ make_args(int largs, PyObject *args, SEXP *e)
 
   for (i=0; i<largs; i++) {
     r = to_Robj(PyTuple_GetItem(args, i));
-    if (!r)
+    if (!r || PyErr_Occurred()	)
       return 0;
     SETCAR(*e, r);
     *e = CDR(*e);
@@ -896,7 +1052,7 @@ make_args(int largs, PyObject *args, SEXP *e)
 
 /* Implements the conversion rules for names. See the 'USING' file. We
    don't care about '<-' because it doesn't appear in keywords. */
-char *
+const char *
 dotter(char *s)
 {
   char *r, *res;
@@ -926,7 +1082,7 @@ int
 make_kwds(int lkwds, PyObject *kwds, SEXP *e)
 {
   SEXP r;
-  char *s;
+  const char *s;
   int i;
   PyObject *citems=NULL, *it;
   PyObject *kwname;
@@ -941,7 +1097,7 @@ make_kwds(int lkwds, PyObject *kwds, SEXP *e)
       goto fail;
     r = to_Robj(PyTuple_GetItem(it, 1));
     Py_DECREF(it);
-    if (!r)
+    if (!r || PyErr_Occurred())
       goto fail;
 
     SETCAR(*e, r);
@@ -957,7 +1113,7 @@ make_kwds(int lkwds, PyObject *kwds, SEXP *e)
       goto fail;
 
     SET_TAG(*e, Rf_install(s));
-    PyMem_Free(s);
+    PyMem_Free( (void*) s);
     *e = CDR(*e);
   }
   Py_XDECREF(citems);
@@ -1025,7 +1181,7 @@ int
 make_argl(int largl, PyObject *argl, SEXP *e)
 {
   SEXP rvalue;
-  char *name;
+  const char *name;
   int i;
   PyObject *it, *nobj, *value;
 
@@ -1064,17 +1220,19 @@ make_argl(int largl, PyObject *argl, SEXP *e)
 
     /* Value can be anything. */
     value = PySequence_GetItem(it, 1);
-    if (!value) 
+    if (!value || PyErr_Occurred() ) 
       {
-        PyMem_Free(name);
+        PyMem_Free( (void*) name);
         goto fail;
       }
 
     rvalue =  to_Robj(value);
     Py_DECREF(value);
-
     Py_DECREF(it);
 
+    if(PyErr_Occurred())
+      goto fail;
+
     /* Add parameter value to call */
     SETCAR(*e, rvalue);
 
@@ -1082,7 +1240,7 @@ make_argl(int largl, PyObject *argl, SEXP *e)
     if (name && strlen(name)>0) 
       {
         SET_TAG(*e, Rf_install(name));
-        PyMem_Free(name);
+        PyMem_Free((void*) name);
       }
 
     /* Move index to new end of call */
@@ -1176,7 +1334,7 @@ Robj_autoconvert(PyObject *self, PyObject *args, PyObject *kwds)
 {
   PyObject *obj;
   int conversion=-2;
-  static char *kwlist[] = {"val", 0};
+  char *kwlist[] = {"val", 0};
 
   if (!PyArg_ParseTupleAndKeywords(args, kwds, "|i:autoconvert", kwlist,
                                   &conversion))
@@ -1202,7 +1360,7 @@ static PyObject *
 Robj_as_py(PyObject *self, PyObject *args, PyObject *kwds)
 {
   PyObject *obj;
-  static char *kwlist[] = {"mode", 0};
+  char *kwlist[] = {"mode", 0};
   int conv=default_mode;
 
   if (!PyArg_ParseTupleAndKeywords(args, kwds, "|i:as_py", kwlist,
@@ -1264,6 +1422,9 @@ Robj_ass_item(PyObject *a, int i, PyObject *v)
   SETCAR(CDR(CDR(e)), ri);
   SETCAR(CDR(CDR(CDR(e))), to_Robj(v));
 
+  if(PyErr_Occurred())
+    return -1;
+
   if (!(robj = do_eval_expr(e))) {
     UNPROTECT(1);
     return -1;
@@ -1530,8 +1691,14 @@ void r_finalize(void)
     CleanEd();              
     KillAllDevices();       
     if((tmpdir = getenv("R_SESSION_TMPDIR"))) {          
-      snprintf((char *)buf, 1024, "rm -rf %s", tmpdir); 
-      R_system((char *)buf);                            
+
+#ifdef _WIN32
+      snprintf((char *)buf, 1024, "rmdir /S /Q %s", tmpdir); 
+#else
+      snprintf((char *)buf, 1024, "rm -rf %s", tmpdir);
+#endif
+
+      R_system((char *)buf);
     }
 
     PrintWarnings();	/* from device close and .Last */
@@ -1556,7 +1723,7 @@ init_numeric(void)
   if(use_numeric)
     {
       import_array();
-      multiarray = PyImport_ImportModule("multiarray");
+      multiarray = PyImport_ImportModule(PY_ARRAY_MODULE_NAME);
       if (multiarray) {
         dict = PyModule_GetDict(multiarray);
         if (dict)
@@ -1681,7 +1848,7 @@ init_embedded_win32( void ) {
   /* run as "interactive", so server won't be killed after an error */
   Rp->R_Slave = Rp->R_Verbose = 0;
   Rp->R_Interactive = TRUE;
-  Rp->RestoreAction = SA_RESTORE; /* no restore */
+  Rp->RestoreAction = SA_NORESTORE; /* no restore */
   Rp->SaveAction    = SA_NOSAVE;  /* no save */
   
 #if R_VERSION < 0x20000   // pre-R-2.0.0
@@ -1721,7 +1888,7 @@ INIT_RPY(void)
 
   if( !strlen(RHOME) || !strlen(RVERSION) || !strlen(RVER) || !strlen(RUSER))
     {
-      PyErr_Format(RPyExc_Exception,
+      PyErr_Format(RPy_Exception,
                    "Unable to load R path or version information");
       return;
     }
@@ -1732,8 +1899,7 @@ INIT_RPY(void)
   Robj_Type.tp_alloc = PyType_GenericAlloc;
 #endif
 
-  //m = Py_InitModule(MacroQuote(RPY_SHNAME), rpy_methods);
-  m = Py_InitModule(RPY_SHNAME, rpy_methods);
+  m = Py_InitModule(xstr(RPY_SHNAME), rpy_methods);
   d = PyModule_GetDict(m);
 
   /* Save this interpreter */
@@ -1756,6 +1922,13 @@ INIT_RPY(void)
                     defaultargv);
 #endif
 
+
+#ifndef CSTACK_DEFNS
+  /* Disable C stack checking, which is incompatible with use as a
+     shared library. */
+  R_CStackLimit = (uintptr_t)-1; 
+#endif
+
   /* Restore Python handlers */
   PyOS_setsig(SIGINT, old_int);
 #ifndef _WIN32
@@ -1763,10 +1936,20 @@ INIT_RPY(void)
   PyOS_setsig(SIGUSR2, old_usr2);
 #endif
 
-  /* The new exception */
-  RPyExc_Exception = PyErr_NewException("rpy.RException", NULL, NULL);
-  if (RPyExc_Exception)
-    PyDict_SetItemString(d, "RException", RPyExc_Exception);
+  /* Several new exceptions: */
+  RPy_Exception               = PyErr_NewException("rpy.RPy_Exception",               NULL,          NULL);
+  RPy_TypeConversionException = PyErr_NewException("rpy.RPy_TypeConversionException", RPy_Exception, NULL);
+  RPy_RException              = PyErr_NewException("rpy.RPy_RException",              RPy_Exception, NULL);
+
+  if (!RPy_Exception || !RPy_TypeConversionException || !RPy_RException )               
+    {
+      PyErr_Format(RPy_Exception, "Unable create RPy exceptions");
+      return;
+    }
+
+  PyDict_SetItemString(d, "RPy_Exception",               RPy_Exception);
+  PyDict_SetItemString(d, "RPy_TypeConversionException", RPy_TypeConversionException);
+  PyDict_SetItemString(d, "RPy_RException",              RPy_RException);
 
   // The conversion table
   class_table = PyDict_New();
@@ -1796,15 +1979,13 @@ INIT_RPY(void)
   // I/O routines
   init_io_routines();
 
-
-
   rpy = PyImport_ImportModule("rpy");
   rpy_dict = PyModule_GetDict(rpy);
   //  r_lock = PyDict_GetItemString(rpy_dict, "_r_lock");
   //  PyObject_Print(r_lock, stderr, Py_PRINT_RAW);
   r_lock = NULL;
 
-  if( Py_AtExit(	r_finalize ) )
+  if( Py_AtExit( r_finalize ) )
     {
       fprintf(stderr, "Warning: Unable to set R finalizer.");
       fflush(stderr);
