@@ -27,6 +27,29 @@ class RobjTestCase(unittest.TestCase):
             #print k, d[k]
             self.failUnless(k == d[k])
             
+    def testBadKeywordParameters(self):
+        def badkwname():
+            # Normally, the python C API catches this, but rpymodule does it's
+            # own keyword arg processing, and inadequate error checking meant
+            # rpy was dereferencing a null pointer.
+            r.list(**{None: 1})
+        self.failUnlessRaises(TypeError, badkwname)
+
+    def testBadKeywordDict(self):
+        # This happens if the user accidently passes an object that is
+        # superficially like a dictionary, but the object returned by it's
+        # items() method raises an exception when fetching items. A missed
+        # error check meant rpymodule was dereferencing a null pointer.
+        class A(dict):
+            def items(self):
+                return B()
+        class B(list):
+            def __getitem__(self, n):
+                raise TypeError
+        def badkwdict():
+            r.list(**A(x=1))
+        self.failUnlessRaises(TypeError, badkwdict)
+
     def testNameConversions(self):
         self.failUnless(r.array is r['array'] and
                         r.print_ is r['print'] and
@@ -34,7 +57,7 @@ class RobjTestCase(unittest.TestCase):
                         r.attr__ is r['attr<-'])
 
     def testNotFound(self):
-        self.failUnlessRaises(RException, lambda: r.foo)
+        self.failUnlessRaises(RPyException, lambda: r.foo)
         
     def testNameLengthOne(self):
         self.failUnless(r.T)
